Stan Math Library: stan/math/fwd/mat/fun/determinant.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

determinant.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_FWD_MAT_FUN_DETERMINANT_HPP
2 #define STAN_MATH_FWD_MAT_FUN_DETERMINANT_HPP
3
4 #include <stan/math/prim/mat/fun/Eigen.hpp>
5 #include <stan/math/fwd/core.hpp>
6 #include <stan/math/fwd/mat/fun/typedefs.hpp>
7 #include <stan/math/prim/mat/fun/multiply.hpp>
8 #include <stan/math/fwd/mat/fun/multiply.hpp>
9 #include <stan/math/prim/mat/fun/inverse.hpp>
10 #include <stan/math/fwd/mat/fun/inverse.hpp>
11 #include <stan/math/prim/mat/err/check_square.hpp>
12 #include <boost/math/tools/promotion.hpp>
13 #include <vector>
14
15 namespace stan {
16 namespace math {
17
18 template<typename T, int R, int C>
19 inline
20 fvar<T>
21 determinant(const Eigen::Matrix<fvar<T>, R, C>& m) {
22 using stan::math::multiply;
23 using stan::math::inverse;
24
25 stan::math::check_square("determinant", "m", m);
26 Eigen::Matrix<T, R, C> m_deriv(m.rows(), m.cols());
27 Eigen::Matrix<T, R, C> m_val(m.rows(), m.cols());
28
29 for (size_type i = 0; i < m.rows(); i++) {
30 for (size_type j = 0; j < m.cols(); j++) {
31 m_deriv(i, j) = m(i, j).d_;
32 m_val(i, j) = m(i, j).val_;
33 }
34 }
35
36 Eigen::Matrix<T, R, C> m_inv = inverse<T>(m_val);
37 m_deriv = multiply(m_inv, m_deriv);
38
39 fvar<T> result;
40 result.val_ = m_val.determinant();
41 result.d_ = result.val_ * m_deriv.trace();
42
43 // FIXME: I think this will overcopy compared to retur fvar<T>(...);
44 return result;
45 }
46 }
47 }
48 #endif
typedefs.hpp
multiply.hpp
core.hpp
stan::math::fvar::d_
T d_
Definition: fvar.hpp:15
stan
Definition: log_sum_exp.hpp:8
stan::math::multiply
Eigen::Matrix< fvar< T >, R1, C1 > multiply(const Eigen::Matrix< fvar< T >, R1, C1 > &m, const fvar< T > &c)
Definition: multiply.hpp:21
multiply.hpp
stan::math::size_type
Eigen::Matrix< double, Eigen::Dynamic, Eigen::Dynamic >::Index size_type
Type for sizes and indexes in an Eigen matrix with double e.
Definition: typedefs.hpp:13
stan::math::inverse
Eigen::Matrix< fvar< T >, R, C > inverse(const Eigen::Matrix< fvar< T >, R, C > &m)
Definition: inverse.hpp:20
stan::math::fvar::val_
T val_
Definition: fvar.hpp:14
stan::math::determinant
fvar< T > determinant(const Eigen::Matrix< fvar< T >, R, C > &m)
Definition: determinant.hpp:21
inverse.hpp
inverse.hpp
Eigen.hpp
stan::math::check_square
bool check_square(const char *function, const char *name, const Eigen::Matrix< T_y, Eigen::Dynamic, Eigen::Dynamic > &y)
Return true if the specified matrix is square.
Definition: check_square.hpp:28
stan::math::fvar
Definition: fvar.hpp:13
check_square.hpp

[ Stan Home Page ] © 2011–2016, Stan Development Team.